<?php

class SambaAiStarsModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        $percentage = Tools::getValue('percentage', 0);
        $percentage = max(0, min(100, (int)$percentage));

        $filledStars = floor($percentage / 20);
        $remainder = $percentage % 20;
        $halfStars = ($remainder >= 10) ? 1 : 0;
        $emptyStars = 5 - $filledStars - $halfStars;

        $this->generateStarImage($filledStars, $halfStars, $emptyStars);
    }

    private function generateStarImage($filledStars, $halfStars, $emptyStars)
    {
        $starSize = 30;
        $padding = 2;
        $numStars = 5;
        $width = ($starSize * $numStars) + ($padding * ($numStars - 1));
        $height = $starSize;
        $image = imagecreatetruecolor($width, $height);

        $white = imagecolorallocate($image, 255, 255, 255);
        $yellow = imagecolorallocate($image, 255, 204, 0);
        $gray = imagecolorallocate($image, 200, 200, 200);

        imagefilledrectangle($image, 0, 0, $width, $height, $white);

        $x = 0;

        for ($i = 1; $i <= $filledStars; $i++) {
            $this->drawReviewStar($image, $x + ($starSize / 2), $height / 2, $starSize / 2, $yellow, 1);
            $x += $starSize + $padding;
        }

        if ($halfStars === 1) {
            $this->drawReviewStar($image, $x + ($starSize / 2), $height / 2, $starSize / 2, $yellow, 0.5, $gray);
            $x += $starSize + $padding;
        }

        for ($i = 1; $i <= $emptyStars; $i++) {
            $this->drawReviewStar($image, $x + ($starSize / 2), $height / 2, $starSize / 2, $gray, 1);
            $x += $starSize + $padding;
        }

        header('Content-Type: image/png');
        imagepng($image);
        imagedestroy($image);
        exit;
    }

    private function drawReviewStar($image, $cx, $cy, $radius, $primaryColor, $fillRatio = 1, $secondaryColor = null)
    {
        $points = [];
        for ($i = 0; $i < 10; $i++) {
            $angle = $i * (M_PI / 5) + M_PI / 2;
            $r = ($i % 2 === 0) ? $radius : $radius / 2.5;
            $points[] = $cx + $r * cos($angle);
            $points[] = $cy - $r * sin($angle);
        }

        if ($fillRatio === 1) {
            imagefilledpolygon($image, $points, 10, $primaryColor);
        } elseif ($fillRatio === 0.5) {
            $halfClip = $cx;

            imagefilledpolygon($image, $points, 10, $primaryColor);

            $maskPoints = [];
            foreach ($points as $index => $coord) {
                if ($index % 2 == 0 && $coord > $halfClip) {
                    $maskPoints[] = $coord;
                    $maskPoints[] = $points[$index + 1];
                }
            }
            $maskPoints[] = $cx;
            $maskPoints[] = $cy;

            imagefilledpolygon($image, $maskPoints, count($maskPoints) / 2, $secondaryColor);
        }
    }


}