<?php
/**
 *  @author    Martin Tomasek
 *  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

include_once('fcommon.php');

class FCategory
{
    private $baseQuery;
    private $id_shop;
    private $id_lang;

    public function __construct($common)
    {
        $this->common = $common;
        $this->id_shop = Configuration::get('SAMBA_SHOP');
        $this->id_lang = Configuration::get('SAMBA_LANG');
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $ID_LANG = Configuration::get('SAMBA_LANG');

        $this->baseQuery = '
            SELECT c.id_parent, c.id_category, c.is_root_category, cl.name, 
                   MAX(CASE WHEN cl.id_lang = ' . (int)$this->id_lang  . ' THEN 100 WHEN cl.id_lang = 0 THEN 99 ELSE -1000 END) as lang_order
            FROM ' . _DB_PREFIX_ . 'category AS c
            INNER JOIN ' . _DB_PREFIX_ . 'category_lang AS cl ON cl.id_category = c.id_category
            WHERE cl.id_lang = ' . (int)$this->id_lang . ' AND cl.id_shop = ' . (int)$this->id_shop. '
            GROUP BY c.id_parent, c.id_category, c.is_root_category, cl.name';
    }

    public function genFeed($filePath = null)
    {
        // Determine output method
        $isCron = Configuration::get('SAMBA_CRON') == 1;
        $xmlWriter = new XMLWriter();
        if ($isCron) {
            // Generate to file
            $xmlWriter->openMemory();
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CATEGORIES');
        } else {
            // Generate live output
            header('Content-Type: text/xml');
            $xmlWriter->openURI('php://output');
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CATEGORIES');
        }

        $children = array();
        $roots = array();
        $nodes = array();
        $cats = Db::getInstance()->ExecuteS($this->baseQuery);
        foreach ($cats as $c) {
            $parent_id = $c['id_parent'];
            $node = array('id' => $c['id_category'], 'name' => $c['name'], 'url' => $this->common->genUrls('category', $c['id_category']));
            $nodes[$c['id_category']] = $node;
            if ($c['is_root_category']) {
                $roots[] = $node;
            }

            if (!array_key_exists($parent_id, $children)) {
                $children[$parent_id] = array();
            }
            $children[$parent_id][] = $node;
        }
        foreach ($roots as $root) {
            $this->common->subtree($xmlWriter, $root['id'], $cattext = array(), $nodes, $children);
        }

        // End the CATEGORIES element and the document
        $xmlWriter->endElement();
        $xmlWriter->endDocument();
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        } else {
            $xmlWriter->flush();
        }

        return $nodes;
    }
    public function getCategoryNodes()
    {
        $categories = [];
        $catss = Db::getInstance()->ExecuteS($this->baseQuery);
        foreach ($catss as $c) {
            $categories[$c['id_category']] = [
                'id' => $c['id_category'],
                'parent_id' => $c['id_parent'],
                'is_root_category' => $c['is_root_category'],
                'name' => $c['name'],
                'url' => $this->common->genUrls('category', $c['id_category'])
            ];

        }
        return $categories;
    }

    private function writeCategoryNode($xmlWriter, $category, $categories)
    {
        $xmlWriter->startElement('CATEGORY');
        $xmlWriter->writeElement('ID', $category['id']);
        $xmlWriter->writeElement('NAME', $category['name']);
        $xmlWriter->writeElement('URL', $category['url']);

        $children = array_filter($categories, function ($cat) use ($category) {
            return $cat['parent_id'] == $category['id'];
        });

        foreach ($children as $child) {
            $this->writeCategoryNode($xmlWriter, $child, $categories);
        }

        $xmlWriter->endElement(); // CATEGORY
    }
}
