<?php
/**
 *  @autor    Martin Tomasek
 *  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

include_once('fcommon.php');

class FCustomer
{
    private $baseQuery;
    private $sambaModuleSms;
    private $sambaModuleNewsletterSms;

    public function __construct($common, $date_from)
    {
        if (empty($date_from)) {
            $date_from = '1970-01-01';
        }
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $this->common = $common;
        $isIncremental = Configuration::get('SAMBA_INCREMENTAL') == 1;
        $this->sambaModuleSms =  Configuration::get('SAMBA_MODULE_SMS');
        $this->sambaModuleNewsletterSms =  Configuration::get('SAMBA_MODULE_NEWSLETTER_SMS');

        $date_from = $isIncremental ? date('Y-m-d H:i:s', strtotime('-7 days')) : $date_from;

        $whereQuery = "WHERE c.email NOT LIKE '%@allegro%' AND c.newsletter = 1";

        if ($this->sambaModuleSms) {
            $whereQuery = "WHERE c.email NOT LIKE '%@allegro%' AND (c.newsletter = 1 OR c.optin = 1)";
        }

        $this->baseQuery = "
            SELECT 
                c.id_customer, c.firstname, c.lastname, c.email, c.newsletter,
                c.newsletter_date_add, c.optin, c.birthday, c.date_add, c.deleted, c.is_guest,
                gl.name AS gender, a.postcode, a.phone, a.phone_mobile, ggl.name AS group_name,
                cc.call_prefix, cc.iso_code, a.id_address AS address_id
            FROM " . _DB_PREFIX_ . "customer c
            LEFT JOIN (
                SELECT ad.id_customer, ad.id_address, ad.postcode, ad.phone, ad.phone_mobile, ad.id_country FROM " . _DB_PREFIX_ . "address ad
                INNER JOIN (
                    SELECT id_customer, MAX(id_address) AS max_address
                    FROM " . _DB_PREFIX_ . "address
                    GROUP BY id_customer
                ) AS sub ON ad.id_customer = sub.id_customer AND ad.id_address = sub.max_address
            ) AS a ON a.id_customer = c.id_customer
            LEFT JOIN " . _DB_PREFIX_ . "country AS cc ON cc.id_country = a.id_country
            LEFT JOIN " . _DB_PREFIX_ . "gender_lang AS gl ON gl.id_gender = c.id_gender AND gl.id_lang = '" . (int)$ID_LANG . "'
            LEFT JOIN " . _DB_PREFIX_ . "group_lang AS ggl ON ggl.id_group = c.id_default_group AND ggl.id_lang = '" . (int)$ID_LANG . "'
        ";

        $this->baseQuery .= " " . $whereQuery;
    }

    public function validatePhoneFormat($phone, $CallPrefix)
    {
        $phone_number = filter_var($phone, FILTER_SANITIZE_NUMBER_INT);

        if (preg_match('/^\+[0-9]{1,2}[0-9]{3}[0-9]{7}$/', $phone_number)) {
            return $phone_number;
        } else if (preg_match('/^[0-9]{9}$/', $phone_number)) {
            return '+' . $CallPrefix . $phone_number;
        }

        return '';
    }

    public function genFeedPageNewsletter($xmlWriter)
    {
        $customers = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "emailsubscription");
        foreach ($customers as $c) {
            $email = $c['email'];
            $check = Db::getInstance()->executeS("SELECT COUNT(email) FROM " . _DB_PREFIX_ . "customer WHERE email = '$email'");
            if (!$check[0]['COUNT(email)']) {
                $phone_mobile = null;

                if (isset($c['phone_mobile'])) {
                    $phone_mobile = $c['phone_mobile'];
                }  elseif (isset($c['phone'])) {
                    $phone_mobile = $c['phone'];
                }

                $phone_mobile_validate = null;

                if (isset($phone_mobile)) {
                    $phone_mobile_validate = $this->validatePhoneFormat($phone_mobile, $c['call_prefix']);
                }

                $xmlWriter->startElement('CUSTOMER');
                $xmlWriter->writeElement('CUSTOMER_ID', 'N' . $c['id']);

                $c['email'] = preg_replace('/\s+/', '', $c['email']);
                $xmlWriter->writeElement('EMAIL', $c['email']);
                $xmlWriter->writeElement('NEWSLETTER_FREQUENCY', 'every day');

                 if (1 == $this->sambaModuleSms && $phone_mobile_validate) {
                    $xmlWriter->writeElement('SMS_FREQUENCY', $c['optin'] ? 'every day' : 'never');
                } else if (1 == $this->sambaModuleNewsletterSms && $phone_mobile_validate) {
                    $xmlWriter->writeElement('SMS_FREQUENCY', 'every day');
                } else {
                    $xmlWriter->writeElement('SMS_FREQUENCY', 'never');
                }

                // if (1 == $this->sambaModuleNewsletterSms && $phone_mobile_validate) {
                //     $xmlWriter->writeElement('SMS_FREQUENCY', 'every day');
                // } else if (1 == $this->sambaModuleSms && $phone_mobile_validate) {
                //     $xmlWriter->writeElement('SMS_FREQUENCY', $c['optin'] ? 'every day' : 'never');
                // } else {
                //     $xmlWriter->writeElement('SMS_FREQUENCY', 'never');
                // }
                
                $xmlWriter->writeElement('REGISTRATION', $this->common->dtIso($c['newsletter_date_add']));
                $xmlWriter->endElement(); // CUSTOMER
            }
        }
    }

    public function genFeed($filePath = null)
    {
        $pageSize = 10000;
        $lastId = 0;
        $batchCount = 0;

        // Determine output method
        $isCron = Configuration::get('SAMBA_CRON') == 1;
        $xmlWriter = new XMLWriter();
        if ($isCron) {
            // Generate to file
            $xmlWriter->openMemory();
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CUSTOMERS');
        } else {
            // Generate live output
            header('Content-Type: text/xml');
            $xmlWriter->openURI('php://output');
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CUSTOMERS');
        }

        // Add customers from emailsubscription table
        $this->genFeedPageNewsletter($xmlWriter);

        while (true) {
            $customers = $this->fetchCustomers($lastId, $pageSize);

            if (empty($customers)) {
                break;
            }

            foreach ($customers as $c) {
                $lastId = $c['id_customer'];

                $phone_mobile = null;

                if (!empty($c['phone_mobile'])) {
                    $phone_mobile = $c['phone_mobile'];
                }  elseif (!empty($c['phone'])) {
                    $phone_mobile = $c['phone'];
                }

                $phone_mobile_validate = null;

                if (!empty($phone_mobile)) {
                    $phone_mobile_validate = $this->validatePhoneFormat($phone_mobile, $c['call_prefix']);
                }

                $dateTimeZone = date("Y-m-d\TH:i:sP", strtotime($c['newsletter_date_add']));

                $xmlWriter->startElement('CUSTOMER');
                $xmlWriter->writeElement('FIRST_NAME', $c['firstname']);
                $xmlWriter->writeElement('LAST_NAME', $c['lastname']);
                $xmlWriter->writeElement('CUSTOMER_ID', $c['id_customer']);
                $xmlWriter->writeElement('EMAIL', $c['email']);

                if (!empty($phone_mobile_validate)) {
                    $xmlWriter->writeElement('PHONE', $phone_mobile_validate);
                }

                if (!empty($c['postcode'])) {
                    $xmlWriter->writeElement('ZIP_CODE', $c['postcode']);
                }

                $xmlWriter->writeElement('NEWSLETTER_FREQUENCY', $c['newsletter'] ? 'every day' : 'never');

                if (1 == $this->sambaModuleSms && $phone_mobile_validate) {
                    $xmlWriter->writeElement('SMS_FREQUENCY', $c['optin'] ? 'every day' : 'never');
                } else if (1 == $this->sambaModuleNewsletterSms && $phone_mobile_validate) {
                    $xmlWriter->writeElement('SMS_FREQUENCY', $c['newsletter'] ? 'every day' : 'never');
                } else {
                    $xmlWriter->writeElement('SMS_FREQUENCY', 'never');
                }

                if ($c['newsletter_date_add'] != '0000-00-00 00:00:00') {
                    $xmlWriter->writeElement('NLF_TIME', $dateTimeZone);
                }

                $xmlWriter->writeElement('REGISTRATION', $this->common->dtIso($c['date_add']));
                $xmlWriter->writeElement('PRICE_CATEGORY', $c['group_name']);
                $xmlWriter->startElement('PARAMETERS');

                if(isset($c['name']) AND !empty($c['name'])){
                    $xmlWriter->startElement('PARAMETER');
                    $xmlWriter->writeElement('NAME', 'Gender');
                    $xmlWriter->writeElement('VALUE', $c['name']);
                    $xmlWriter->endElement(); // PARAMETER
                }

                if ($c['birthday'] != '0000-00-00' && $c['birthday'] != '') {
                    $xmlWriter->startElement('PARAMETER');
                    $xmlWriter->writeElement('NAME', 'Birthday');
                    $xmlWriter->writeElement('VALUE', $c['birthday']);
                    $xmlWriter->endElement(); // PARAMETER
                }

                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', "Imie");
                $xmlWriter->writeElement('VALUE', $c['firstname']);
                $xmlWriter->endElement(); // PARAMETER
                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', 'Kraj');
                $xmlWriter->writeElement('VALUE', $c['iso_code']);
                $xmlWriter->endElement(); // PARAMETER
                $xmlWriter->endElement(); // PARAMETERS
                $xmlWriter->endElement(); // CUSTOMER
            }

            // Flush the data to the file every batch
            if ($isCron && ++$batchCount % 10 == 0) {
                file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
            }

            // Break if fewer customers than page size, indicating last batch
            if (count($customers) < $pageSize) {
                break;
            }
        }

        // Final flush to make sure we haven't missed anything
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        }

        // End the CUSTOMERS element and the document
        $xmlWriter->endElement();
        $xmlWriter->endDocument();
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        } else {
            $xmlWriter->flush();
        }
    }

    private function fetchCustomers($lastId, $limit)
    {
        $query = $this->baseQuery . " AND c.id_customer > $lastId ORDER BY c.date_add ASC LIMIT $limit";
        $customers = Db::getInstance()->executeS($query);
        return $customers;
    }
}
