<?php
include_once('fcommon.php');
include_once('fcategory.php');

class FProduct {
    private $baseQuery;

    private $common;

    private $review;

    private $starLink;

    public function __construct($common)
    {
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $this->common = $common;
        $this->baseQuery = '
        SELECT 
                    p.*, 
                    i.id_image as image, 
                    pl.*, 
                    cl.link_rewrite as cat_link_rewrite, 
                    s.quantity as stock, 
                    mfg.name as mfg_desc 
                FROM ' . _DB_PREFIX_ . 'product AS p
                LEFT JOIN ' . _DB_PREFIX_ . 'product_lang AS pl ON pl.id_product = p.id_product
                LEFT JOIN ' . _DB_PREFIX_ . 'image AS i ON p.id_product = i.id_product
                LEFT JOIN ' . _DB_PREFIX_ . 'stock_available AS s ON s.id_product = p.id_product
                LEFT JOIN ' . _DB_PREFIX_ . 'product_shop AS ps ON ps.id_product = p.id_product
                LEFT JOIN ' . _DB_PREFIX_ . 'category_lang AS cl ON cl.id_category = p.id_category_default
                LEFT JOIN ' . _DB_PREFIX_ . 'manufacturer AS mfg ON mfg.id_manufacturer = p.id_manufacturer
                LEFT JOIN (
                    SELECT 
                        id_product, 
                        MIN(position) AS position 
                    FROM ' . _DB_PREFIX_ . 'image 
                    GROUP BY id_product
                ) img_pos ON img_pos.id_product = i.id_product AND img_pos.position = i.position 
                WHERE 
                    pl.id_lang = ' . $ID_LANG . ' 
                    AND ps.id_shop = ' . $ID_SHOP . ' 
                    AND s.id_product_attribute = 0 
                    AND cl.id_lang = ' . $ID_LANG . ' 
                GROUP BY 
                    p.id_product
        ';

        if(Configuration::get('SAMBA_REVIEW') && Configuration::get('SAMBA_REVIEW_URL')) {
            $this->review = $this->extractProductReviews(Configuration::get('SAMBA_REVIEW_URL'), Configuration::get('SAMBA_REVIEW_MAX'));
            $this->starLink = Context::getContext()->link->getModuleLink('sambaai', 'stars', ['percentage' => '']);
        }


    }

    public function genFeed($filePath = null) {
        $pageSize = 10000;
        $batchCount = 0;

        // Determine output method
        $isCron = Configuration::get('SAMBA_CRON') == 1;
        $xmlWriter = new XMLWriter();
        if ($isCron) {
            // Generate to file
            $xmlWriter->openMemory();
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('PRODUCTS');
        } else {
            // Generate live output
            header('Content-Type: text/xml');
            $xmlWriter->openURI('php://output');
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('PRODUCTS');
        }

        $categories = new FCategory($this->common);
        $cats = $categories->getCategoryNodes();

        $page = 0;
        while (true) {
            $count = $this->genFeedPage($page, $cats, $xmlWriter);
            if ($count < $pageSize) {
                break;
            }
            $page++;
            if ($isCron && ++$batchCount % 10 == 0) {
                file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
            }
        }

        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        }

        $xmlWriter->endElement();
        $xmlWriter->endDocument();
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        } else {
            $xmlWriter->flush();
        }
    }

    private function genFeedPage($page, $cats, $xmlWriter) {
        $pagesize = 10000;
        $products = Db::getInstance()->ExecuteS($this->baseQuery . ' ORDER BY p.id_product LIMIT ' . $pagesize . ' OFFSET ' . $page * $pagesize);
        $productsVariants = Configuration::get('SAMBA_PRODUCTS_VARIANTS');
        $this->genProducts($products, $productsVariants, $cats, $xmlWriter);
        return count($products);
    }

    private function genProducts($products, $productsVariants, $cats, $xmlWriter) {
        foreach ($products as $p) {
            if ($productsVariants) {
                $this->genProductWithVariantsAsProducts($p, $cats, $xmlWriter);
            } else {
                $this->genProductWithEmbeddedVariants($p, $cats, $xmlWriter);
            }
        }
    }

    private function genProductWithVariantsAsProducts($p, $cats, $xmlWriter) {
        $product_id = $p['id_product'];
        $product = new Product((int)$product_id);
        if (!$product->hasAttributes()) {
            $this->genProduct($p, $cats, $xmlWriter);
        } else {
            $this->genVariants($product_id, $p, $cats, $xmlWriter);
        }
    }

    private function genProductWithEmbeddedVariants($p, $cats, $xmlWriter) {
        $product_id = $p['id_product'];
        $product = new Product((int)$product_id);

        $xmlWriter->startElement('PRODUCT');
        $this->writeProductBaseData($p, $cats, $xmlWriter);

        if ($product->hasAttributes()) {
            $this->writeVariants($product_id, $p, $cats, $xmlWriter);
        }

        $xmlWriter->endElement(); // PRODUCT
    }

    private function genProduct($p, $cats, $xmlWriter) {
        $xmlWriter->startElement('PRODUCT');
        $this->writeProductBaseData($p, $cats, $xmlWriter);
        $xmlWriter->endElement();
    }

    private function getCategoryPath($node_id, $nodes) {
        $cattext = array();

        $buildPath = function ($node_id, &$cattext, $nodes) use (&$buildPath) {
            if (!isset($nodes[$node_id])) {
                return;
            }

            $node = $nodes[$node_id];
            array_unshift($cattext, $node['name']);

            if ($node['parent_id'] && isset($nodes[$node['parent_id']])) {
                $buildPath($node['parent_id'], $cattext, $nodes);
            }
        };

        $buildPath($node_id, $cattext, $nodes);

        if (!empty($cattext)) {
            array_shift($cattext);
        }

        // Remove 'Strona główna' if it exists
        $cattext = array_filter($cattext, function ($category) {
            return $category !== 'Główna';
        });

        $cattext = array_filter($cattext, function ($category) {
            return $category !== 'Nasze kolekcje';
        });

        $cattext = array_filter($cattext, function ($category) {
            return $category !== 'Strona główna';
        });

        $cattext = array_values($cattext);
        if(empty($cattext)) {
            return 'Ogólne';
        }
        return implode(' | ', $cattext);
    }

    private function writeProductBaseData($p, $cats, $xmlWriter) {
        $common = $this->common;
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $genGroups = Configuration::get('SAMBA_PRODUCTS_GROUP_PRICE');
        $EMPLOYEE_ID = $this->common->getConfig('EMPLOYEE_ID');
        $sGroups = Group::getGroups($ID_LANG);
        $context = Context::getContext();
        $employee = new Employee($EMPLOYEE_ID, $ID_LANG, $ID_SHOP);
        $context->employee = $employee;

        $product_id = $p['id_product'];
        $xmlWriter->writeElement('PRODUCT_ID', $product_id . '-0');
        $xmlWriter->writeElement('TITLE', $p['name']);
        $xmlWriter->writeElement('DESCRIPTION', 'BRAK');
        $xmlWriter->writeElement('URL', $common->genUrls('product', $product_id));
        $xmlWriter->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $p['link_rewrite']));
        if (isset($p['visibility']) && ($p['visibility'] === 'search' || $p['visibility'] === 'none')) {
            $xmlWriter->writeElement('SHOW', 'FALSE');
        } else {
            $xmlWriter->writeElement('SHOW', 'TRUE');
        }

        $categoryPath = $this->getCategoryPath($p['id_category_default'], $cats);
        $xmlWriter->writeElement('CATEGORYTEXT', $categoryPath ?? 'Ogólne');


        // Adding Product Features
        $aGroups = $this->getGroupPrice($sGroups, $product_id, $context);
        if (isset($aGroups) && $genGroups) {
            $xmlWriter->startElement('PRICES');
            foreach ($aGroups as $aGroup) {
                $xmlWriter->startElement('CATEGORY');
                $xmlWriter->writeElement('NAME', $aGroup['name']);
                $xmlWriter->writeElement('PRICE', $aGroup['price']);
                $xmlWriter->writeElement('PRICE_BEFORE_DISCOUNT', $this->price_with_tax((int)$product_id, null, false));
                $xmlWriter->endElement();
            }
            $xmlWriter->endElement();
        }

        $frontfeatures = Product::getFrontFeaturesStatic(1, $product_id);

        $xmlWriter->startElement('PARAMETERS');
        if (isset($frontfeatures)) {
            foreach ($frontfeatures as $feature) {
                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', $feature['name']);
                $xmlWriter->writeElement('VALUE', $feature['value']);
                $xmlWriter->endElement();
            }
        }
        if (isset($this->review[$p['reference']])) {
            $reviewCounter = 1;
            $totalRating = 0;
            $reviewCount = 0;
            $maxRating = 5;

            foreach ($this->review[$p['reference']] as $review) {
                $totalRating += $review['rating'];
                $reviewCount++;

                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', "Opinia nr $reviewCounter - treść:");
                $xmlWriter->writeElement('VALUE', $review['content']);
                $xmlWriter->endElement();

                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', "Opinia nr $reviewCounter - ocena:");
                $xmlWriter->writeElement('VALUE', $review['rating']);
                $xmlWriter->endElement();

                $reviewCounter++;
            }

            if ($reviewCount > 0) {
                $averageRating = $totalRating / $reviewCount;
                $roundedRating = round($averageRating * 2) / 2;
                $percentageValue = ($averageRating / $maxRating) * 100;
            } else {
                $roundedRating = 0;
                $percentageValue = 0;
            }

            $xmlWriter->startElement('PARAMETER');
            $xmlWriter->writeElement('NAME', "Ocena GMC - grafika:");
            $xmlWriter->writeElement('VALUE', $this->starLink . $percentageValue);
            $xmlWriter->endElement();

            $xmlWriter->startElement('PARAMETER');
            $xmlWriter->writeElement('NAME', "Ocena GMC - liczba:");
            $xmlWriter->writeElement('VALUE', number_format($roundedRating, 1));
            $xmlWriter->endElement();
        }
        $xmlWriter->endElement();

        $stock = $p['stock'];
        if (!$p['active']) {
            $stock = 0;
        }
        if ($p['visibility'] == 'none') {
            $stock = 0;
        }
        if ($p['show_price'] <= 0) {
            $stock = 0;
        }
        if ($stock < 0) {
            $stock = 0;
        }
        $xmlWriter->writeElement('PRICE', $this->price_with_tax((int)$product_id, null, true));
        $xmlWriter->writeElement('PRICE_BEFORE_DISCOUNT', $this->price_with_tax((int)$product_id, null, false));

        $wsp = $p['wholesale_price'];
        if ($wsp && $wsp > 0) {
            $xmlWriter->writeElement('PRICE_BUY', (int)((float)$wsp * 100) / 100);
        } // buy price rounded to 2 decimal places

        $xmlWriter->writeElement('BRAND', $p['mfg_desc']);
        $xmlWriter->writeElement('STOCK', $stock);
    }

    private function writeVariants($product_id, $p, $cats, $xmlWriter) {
        $db = Db::getInstance();
        $common = $this->common;
        $q = 'SELECT pa.*, pac.*, sa.quantity as stock 
              FROM ' . $common->tname('product_attribute', 'pa') . ' 
              ' . $common->tjoin('product_attribute_combination', 'pac') . ' ON pa.id_product_attribute = pac.id_product_attribute 
              ' . $common->tjoin('stock_available', 'sa') . ' ON pa.id_product_attribute = sa.id_product_attribute 
              WHERE pa.id_product = ' . $product_id . ' 
              ORDER BY pac.id_product_attribute';
        $variants = $db->ExecuteS($q);

        if (!$variants) {
            return false;
        }

        foreach ($common->groupby($variants, 'id_product_attribute') as $id => $vg) {
            if (empty($vg)) {
                continue;
            }
            $id_product_attribute = $vg[0]['id_product_attribute'];
            $variant_id = $product_id . '-' . $id_product_attribute;

            $xmlWriter->startElement('VARIANT');
            $this->writeVariantBaseData($variant_id, $product_id, $id_product_attribute, $p, $vg[0]['stock'], $cats, $xmlWriter);
            $xmlWriter->endElement(); // VARIANT
        }

        return true;
    }

    private function genVariants($product_id, $p, $cats, $xmlWriter) {
        $db = Db::getInstance();
        $common = $this->common;
        $q = 'SELECT pa.*, pac.*, sa.quantity as stock 
              FROM ' . $common->tname('product_attribute', 'pa') . ' 
              ' . $common->tjoin('product_attribute_combination', 'pac') . ' ON pa.id_product_attribute = pac.id_product_attribute 
              ' . $common->tjoin('stock_available', 'sa') . ' ON pa.id_product_attribute = sa.id_product_attribute 
              WHERE pa.id_product = ' . $product_id . ' 
              ORDER BY pac.id_product_attribute';
        $variants = $db->ExecuteS($q);

        if (!$variants) {
            return false;
        }

        foreach ($common->groupby($variants, 'id_product_attribute') as $id => $vg) {
            if (empty($vg)) {
                continue;
            }
            $id_product_attribute = $vg[0]['id_product_attribute'];
            $variant_id = $product_id . '-' . $id_product_attribute;

            $xmlWriter->startElement('PRODUCT');
            $this->writeVariantBaseData($variant_id, $product_id, $id_product_attribute, $p, $vg[0]['stock'], $cats, $xmlWriter);
            $xmlWriter->endElement(); // PRODUCT
        }

        return true;
    }

    private function writeVariantBaseData($variant_id, $product_id, $id_product_attribute, $p, $stock, $cats, $xmlWriter) {
        $common = $this->common;

        $xmlWriter->writeElement('PRODUCT_ID', $variant_id);
        $xmlWriter->writeElement('TITLE', $p['name']);
        $xmlWriter->writeElement('DESCRIPTION', 'BRAK');
        $xmlWriter->writeElement('URL', $common->genUrls('variant', $product_id, $id_product_attribute));
        $xmlWriter->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $p['link_rewrite']));

        if (isset($p['visibility']) && ('search' === $p['visibility'] || 'none' === $p['visibility'])) {
            $xmlWriter->writeElement('SHOW', 'FALSE');
        } else {
            $xmlWriter->writeElement('SHOW', 'TRUE');
        }

        $categoryPath = $this->getCategoryPath($p['id_category_default'], $cats);
        $xmlWriter->writeElement('CATEGORYTEXT', $categoryPath ?? 'Ogólne');

        // Add variant-specific prices
        $xmlWriter->writeElement('PRICE', $this->price_with_tax((int)$product_id, (int)$id_product_attribute, true));
        $xmlWriter->writeElement('PRICE_BEFORE_DISCOUNT', $this->price_with_tax((int)$product_id, (int)$id_product_attribute, false));
        if ($p['wholesale_price'] > 0) {
            $xmlWriter->writeElement('PRICE_BUY', $p['wholesale_price']);
        }

        if ($stock < 0) {
            $stock = 0;
        }
        $xmlWriter->writeElement('STOCK', $stock);
    }

    private function getGroupPrice($groups, $id_product, $context, $id_product_attribute = null) {
        $aGroup = array();
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $ID_CURRENCY = Currency::getDefaultCurrency()->id;
        $ID_COUNTRY = $context->country->id;
        foreach ($groups as $group) {
            $reduction_value = 0;
            $price = Product::getPriceStatic($id_product, true, $id_product_attribute, 2, null, false, null);
            $today = date('y-m-d');
            $reduction = '';
            $spec_reductions = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "specific_price
                WHERE
                id_shop = '$ID_SHOP' AND
                id_product = $id_product AND
                id_product_attribute = '" . $id_product_attribute . "'
            ");
            array_multisort(
                array_map(
                    static function ($element) {
                        return $element['reduction'];
                    },
                    $spec_reductions
                ),
                SORT_DESC,
                $spec_reductions
            );
            $id_group = $group['id_group'];
            if ($group["reduction"] > 0) {
                if ($reduction < $group['reduction']) {
                    $reduction = ($group['reduction'] / 100);
                    $reduction_value = $price * ($reduction);
                    $aGroup[$id_group]['reduction'] = $reduction;
                    $aGroup[$id_group]['name'] = $group['name'];
                    $aGroup[$id_group]['reduction_value'] = $reduction_value;
                    $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
                }
            } else {
                $aGroup[$id_group]['reduction'] = '';
                $aGroup[$id_group]['name'] = $group['name'];
                $aGroup[$id_group]['reduction_value'] = '';
                $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
            }
            foreach ($spec_reductions as $spec_reduction) {
                if ((strtotime($spec_reduction['to']) > time() && strtotime($spec_reduction['from']) < time()) || $spec_reduction['to'] == '0000-00-00 00:00:00') {
                    if ($spec_reduction['id_group'] == $id_group || $spec_reduction['id_group'] == 0) {
                        if ($spec_reduction['price'] > 0) {
                            $aGroup[$id_group]['price'] = round($spec_reduction['price'] * (1 + (Tax::getProductTaxRate($id_product) / 100)), 2);
                        } elseif ($spec_reduction['reduction_type'] == 'percentage') {
                            $reduction_cache = $price * $spec_reduction['reduction'];
                            if ($reduction_cache > $reduction_value) {
                                $reduction = $spec_reduction['reduction'];
                                $reduction_value = $reduction_cache;
                                $aGroup[$id_group]['reduction'] = $reduction;
                                $aGroup[$id_group]['name'] = $group['name'];
                                $aGroup[$id_group]['reduction_value'] = $reduction_value;
                                $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
                            }
                        } elseif ($spec_reduction['reduction_type'] == 'amount') {
                            if ($spec_reduction['reduction'] > $reduction_value) {
                                $reduction_value = $spec_reduction['reduction'];
                                if ($spec_reduction['reduction_tax'] == 0)
                                    $reduction_value *= (1 + (Tax::getProductTaxRate($id_product) / 100));
                                $aGroup[$id_group]['reduction'] = $reduction;
                                $aGroup[$id_group]['name'] = $group['name'];
                                $aGroup[$id_group]['reduction_value'] = $reduction_value;
                                $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
                            }
                        }
                    }
                }
            }
        }
        return $aGroup;
    }

    private function price_with_tax($id_product, $id_product_attribute, $use_reduction) {
        $specific_price = 0;
        return Product::getPriceStatic(
            (int)$id_product,
            true,
            (int)$id_product_attribute,
            2,
            null,
            false,
            $use_reduction,
            1,
            false,
            null,
            null,
            null,
            $specific_price,
            true,
            true,
            null,
            false
        );
    }

    function extractProductReviews($xmlFilePath, $maxReviewsPerProduct = null) {
        $reviewsByProduct = [];

        $reader = new XMLReader();
        if (!$reader->open($xmlFilePath)) {
            throw new Exception("Unable to open XML file: $xmlFilePath");
        }

        while ($reader->read()) {
            if ($reader->nodeType == XMLReader::ELEMENT && $reader->localName == 'review') {
                $reviewNode = new SimpleXMLElement($reader->readOuterXML());

                $reviewId = (string) $reviewNode->review_id;
                $title = (string) $reviewNode->title;
                $content = (string) $reviewNode->content;
                $date = (string) $reviewNode->review_timestamp;
                $rating = (int) $reviewNode->ratings->overall;

                // Check if products exist and are iterable
                if (isset($reviewNode->products->product) && is_iterable($reviewNode->products->product)) {
                    foreach ($reviewNode->products->product as $product) {
                        // Check if product IDs and SKUs exist and are iterable
                        if (isset($product->product_ids->skus->sku) && is_iterable($product->product_ids->skus->sku)) {
                            foreach ($product->product_ids->skus->sku as $sku) {
                                $sku = trim((string) $sku);

                                if (!isset($reviewsByProduct[$sku])) {
                                    $reviewsByProduct[$sku] = [];
                                }

                                $reviewsByProduct[$sku][] = [
                                    'review_id' => $reviewId,
                                    'title' => $title,
                                    'content' => $content,
                                    'date' => $date,
                                    'rating' => $rating,
                                ];

                                // Sort and limit reviews
                                if ($maxReviewsPerProduct !== null) {
                                    usort($reviewsByProduct[$sku], function ($a, $b) {
                                        return strtotime($b['date']) - strtotime($a['date']);
                                    });

                                    if (count($reviewsByProduct[$sku]) > $maxReviewsPerProduct) {
                                        array_pop($reviewsByProduct[$sku]);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        $reader->close();

        return $reviewsByProduct;
    }
}